
local Get = Object.GetProperty
local Set = Object.SetProperty
local Find = Object.GetNearbyObjects

local TreeTypes = { "Tree", "SpookyTree", "SnowyConiferTree", "CactusTree", "SakuraTree", "PalmTree", "Bush" }

local timeSign = 0
local timeTot = 0
local Direction = 0
local OnRoute = false
local tmpID = 0
local UpdateTrackSigns = true

function Create()
	Set(this,"Wagons",0)
	Set(this,"UpdateTrackSigns",true)
	this.Tooltip = "tooltip_TrainSpawner"
end

function Update(timePassed)
	if timePerUpdate == nil then
		if not this.RailwayCreated then
			Interface.AddComponent(this,"CreateRailway", "Button", "tooltip_button_CreateRailway")
			CheckForExistingSpawners()
		else
			if		 this.Or.x == 0	 and this.Or.y == 1	 then Direction = 1		-- train goes from top to bottom
			elseif	 this.Or.x == 0	 and this.Or.y == -1 then Direction = 2		-- train goes from bottom to top
			elseif	 this.Or.x == 1	 and this.Or.y == 0	 then Direction = 3		-- train goes from left to right
			elseif	 this.Or.x == -1 and this.Or.y == 0	 then Direction = 4		-- train goes from right to left
			end
			Interface.AddComponent(this,"toggleTrainWagons", "Button", "tooltip_button_TrainWagons",this.Wagons,"X")
			Interface.AddComponent(this,"DeleteRailway", "Button", "tooltip_button_DeleteRailway")
			Set(this,"Tooltip","Track: "..this.Id.u)
		end
		timePerUpdate = 1 / World.TimeWarpFactor
		timePerSignUpdate = 1 / World.TimeWarpFactor
	end
	
	timeTot = timeTot + timePassed
	if timeTot >= timePerUpdate then
		timeTot = 0
		
		if this.SpawnTrain == true then
			if UpdateTrackSigns == true or this.UpdateTrackSigns == true then
				BuildTrackSignList()
			end
			if #MyTrackSigns > 0 then SetTrackSignsToGreen() end
			newTrain = Object.Spawn("Train1",this.Pos.x,this.Pos.y)
			newTrain.Or.x = this.Or.x
			newTrain.Or.y = this.Or.y
			Set(newTrain,"WagonsToSpawn",this.Wagons)
			this.SpawnTrain = false
		end
	end
	
	if OnRoute == true then
		timeSign = timeSign + timePassed
		if timeSign >= timePerSignUpdate then
			timeSign = 0
			SetTrackSignsToRed()
			if not Exists(newTrain) then
				OnRoute = false
			end
		end
	end
end

function BuildTrackSignList()
	print("BuildTrackSignList")
	local total = 0
	local mytotal = 0
	MyTrackSigns = {}
	local RailwayTrackSigns = Find(this,"RailwayTrackSign",10000)
	if next(RailwayTrackSigns) then
		for thatSign, dist in pairs(RailwayTrackSigns) do
			total = total + 1
			if IsMyTrackID(thatSign) then
				table.insert(MyTrackSigns,thatSign)
				thatSign.SubType = 0
			end
		end
		print("Total signs: "..total)
	end
	print("My signs: "..#MyTrackSigns)
	RailwayTrackSigns = nil
	UpdateTrackSigns = false
	Set(this,"UpdateTrackSigns",false)
end

function SetTrackSignsToGreen()
	for i = 1,#MyTrackSigns do
		if Exists(MyTrackSigns[i]) then
			MyTrackSigns[i].SubType = 1
			OnRoute = true
		else
			UpdateTrackSigns = true
		end
	end
end

function SetTrackSignsToRed()
	for i = 1,#MyTrackSigns do
		if Exists(MyTrackSigns[i]) then
			if TrainAtSign(MyTrackSigns[i]) then
				MyTrackSigns[i].SubType = 0
			end
		else
			UpdateTrackSigns = true
		end
	end
end

function IsMyTrackID(theSign)
	if theSign.TrackID ~= nil and theSign.TrackID == this.Id.u then return true else return false end
end

function TrainAtSign(theSign)
	if		 Direction == 1 and (newTrain.Pos.y == nil or newTrain.Pos.y >= theSign.Pos.y) then return true
	elseif	 Direction == 2 and (newTrain.Pos.y == nil or newTrain.Pos.y <= theSign.Pos.y) then return true
	elseif	 Direction == 3 and (newTrain.Pos.y == nil or newTrain.Pos.x >= theSign.Pos.x) then return true
	elseif	 Direction == 4 and (newTrain.Pos.y == nil or newTrain.Pos.x <= theSign.Pos.x) then return true
	end
	return false
end

function toggleTrainWagonsClicked()
	this.Wagons = this.Wagons + 1
	if this.Wagons > 10 then this.Wagons = 0 end
	this.SetInterfaceCaption("toggleTrainWagons", "tooltip_button_TrainWagons",this.Wagons,"X")
end

function CheckForExistingSpawners()
	print("CheckForExistingSpawners")
	if		 this.Or.x == 0	 and this.Or.y == 1	 then this.Pos.y = World.NumCellsY-0.5	-- teleport to end of track
	elseif	 this.Or.x == 0	 and this.Or.y == -1 then this.Pos.y = 0.5					-- to check for existing Spawners
	elseif	 this.Or.x == 1	 and this.Or.y == 0	 then this.Pos.x = World.NumCellsX-0.5	-- and delete myself when
	elseif	 this.Or.x == -1 and this.Or.y == 0	 then this.Pos.x = 0.5					-- opposite track already exists
	end
	for existingSpawners in next, Find(this,"TrainSpawner",2) do					 	-- there must be at least 1 tile free between 2 tracks
		if existingSpawners.Id.u ~= this.Id.u then this.Delete() end					-- to be able to put gravel between both tracks.
	end
	if		 this.Or.x == 0 and	 this.Or.y == 1	 then this.Pos.y = 0.5					-- train goes from top to bottom
	elseif	 this.Or.x == 0 and	 this.Or.y == -1 then this.Pos.y = World.NumCellsY-0.5	-- train goes from bottom to top
	elseif	 this.Or.x == 1 and	 this.Or.y == 0	 then this.Pos.x = 0.5					-- train goes from left to right
	elseif	 this.Or.x == -1 and this.Or.y == 0	 then this.Pos.x = World.NumCellsX-0.5	-- train goes from right to left
	end
	for existingSpawners in next, Find(this,"TrainSpawner",2) do
		if existingSpawners.Id.u ~= this.Id.u then this.Delete() end
	end
end

function CreateRailwayClicked()
	print("CreateRailway")
	if (this.Or.x == 0 and this.Or.y == 1) or (this.Or.x == 0 and this.Or.y == -1) then	-- create vertical railway
		local endY = -1
		local PosX = this.Pos.x
		local foundEndY = false
		MyChecker = Object.Spawn("RailwayTrackChecker",PosX,1)
		local CheckerPriorPosY = MyChecker.Pos.y
		while foundEndY == false do
			endY = endY+1
			local myCell = World.GetCell(PosX,endY)
			if myCell.Mat == nil then
				foundEndY = true
				endY = endY-1
				RemoveTrees(MyChecker,4)
				MyChecker.Delete()
				if this.Or.x == 0 and this.Or.y == 1 then
					local newSign = Object.Spawn("RailwayTrackSign",PosX-1.5,endY-0.5)
					newSign.Or.x = this.Or.x
					newSign.Or.y = this.Or.y
					newSign.TrackID = this.Id.u
					Set(newSign,"Tooltip","Track: "..this.Id.u)
					local myCell = World.GetCell(PosX-1.5,endY-1)
					-- myCell.Mat = "FX2K_Tile1"
				else
					local newSign = Object.Spawn("RailwayTrackSign",PosX+1.5,1.5)
					newSign.Or.x = this.Or.x
					newSign.Or.y = this.Or.y
					newSign.TrackID = this.Id.u
					Set(newSign,"Tooltip","Track: "..this.Id.u)
					local myCell = World.GetCell(PosX+1.5,1)
					-- myCell.Mat = "FX2K_Tile1"
				end
			else
				MyChecker.Pos.y = endY
				if MyChecker.Pos.y >= CheckerPriorPosY+3 then
					CheckerPriorPosY = MyChecker.Pos.y
					RemoveTrees(MyChecker,4)
				end
				local myCTL = World.GetCell(PosX-2,endY)
				local myRML = World.GetCell(PosX-1,endY)
				local myRMR = World.GetCell(PosX,endY)
				local myCTR = World.GetCell(PosX+1,endY)
				
				if myCTL.Mat == "RailwayTop" then			-- horizontal rails detected
					local myRML = World.GetCell(PosX-1,endY)
					local myRMR = World.GetCell(PosX,endY)
					myRML.Mat="RailwayCrossing1"; myRMR.Mat="RailwayCrossing2"
					if this.Or.x == 0 and this.Or.y == 1 then
						local newSign = Object.Spawn("RailwayTrackSign",PosX-1.5,endY-0.5)
						newSign.Or.x = this.Or.x
						newSign.Or.y = this.Or.y
						newSign.TrackID = this.Id.u
						Set(newSign,"Tooltip","Track: "..this.Id.u)
						if TrackDirection(PosX,endY,"horizontal") == "ToLeft" then
							local newSign = Object.Spawn("RailwayTrackSign",PosX+1.5,endY-0.5)
							newSign.Or.x = -1
							newSign.Or.y = 0
							newSign.TrackID = tmpID
							Set(newSign,"Tooltip","Track: "..tmpID)
						else
							local newSign = Object.Spawn("RailwayTrackSign",PosX-1.5,endY+2.5)
							newSign.Or.x = 1
							newSign.Or.y = 0
							newSign.TrackID = tmpID
							Set(newSign,"Tooltip","Track: "..tmpID)
						end
					end
				elseif myCTL.Mat == "RailwayBottom" then			-- horizontal rails detected
					local myRML = World.GetCell(PosX-1,endY)
					local myRMR = World.GetCell(PosX,endY)
					myRML.Mat="RailwayCrossing3"; myRMR.Mat="RailwayCrossing4"
					if this.Or.x == 0 and this.Or.y == -1 then
						local newSign = Object.Spawn("RailwayTrackSign",PosX+1.5,endY+1.5)
						newSign.Or.x = this.Or.x
						newSign.Or.y = this.Or.y
						newSign.TrackID = this.Id.u
						Set(newSign,"Tooltip","Track: "..this.Id.u)
						if TrackDirection(PosX,endY,"horizontal") == "ToLeft" then
							local newSign = Object.Spawn("RailwayTrackSign",PosX+1.5,endY-1.5)
							newSign.Or.x = -1
							newSign.Or.y = 0
							newSign.TrackID = tmpID
							Set(newSign,"Tooltip","Track: "..tmpID)
						else
							local newSign = Object.Spawn("RailwayTrackSign",PosX-1.5,endY+1.5)
							newSign.Or.x = 1
							newSign.Or.y = 0
							newSign.TrackID = tmpID
							Set(newSign,"Tooltip","Track: "..tmpID)
						end
					end
					
				elseif myCTL.Mat == "outerlineT" or myCTL.Mat == "verticallinewithzebraT" then		-- horizontal road detected which was created with "outerlineT"
					myCTL.Mat="outerlineT"; myRML.Mat="RailwayCrossingV1"; myRMR.Mat="RailwayCrossingV2"; myCTR.Mat="outerlineT"
					local myCTL = World.GetCell(PosX-2,endY-1)
					local myRML = World.GetCell(PosX-1,endY-1)
					local myRMR = World.GetCell(PosX,endY-1)
					local myCTR = World.GetCell(PosX+1,endY-1)
					myCTL.Mat="ConcreteTiles"; myRML.Mat="RailwayCTileLeft"; myRMR.Mat="RailwayCTileRight"; myCTR.Mat="ConcreteTiles"
					
				elseif myCTL.Mat == "Road" or myCTL.Mat == "verticalzebra" or myCTL.Mat == "dashedcenterline_horizontal" or myCTL.Mat == "solidcenterline_horizontal" or myCTL.Mat == "solidyellowcenterline_horizontal" or myCTL.Mat == "manhole" then
					myCTL.Mat="Road"; myRML.Mat="RailwayRoadLeft"; myRMR.Mat="RailwayRoadRight"; myCTR.Mat="Road"
					
				elseif myCTL.Mat == "outerlineB" or myCTL.Mat == "verticallinewithzebraB" then
					myCTL.Mat="outerlineB"; myRML.Mat="RailwayCrossingV3"; myRMR.Mat="RailwayCrossingV4"; myCTR.Mat="outerlineB"
					local myCTL = World.GetCell(PosX-2,endY+1)
					local myRML = World.GetCell(PosX-1,endY+1)
					local myRMR = World.GetCell(PosX,endY+1)
					local myCTR = World.GetCell(PosX+1,endY+1)
					myCTL.Mat="ConcreteTiles"; myRML.Mat="RailwayCTileLeft"; myRMR.Mat="RailwayCTileRight"; myCTR.Mat="ConcreteTiles"
					endY = endY+1					
					
				elseif myCTL.Mat == "outercurbT" or myCTL.Mat == "verticalcurbwithzebraT" or myCTL.Mat == "curbgutter" then		-- horizontal road detected which was created with "outercurbT"
					myCTL.Mat="outercurbT"; myRML.Mat="RailwayCrossingC1"; myRMR.Mat="RailwayCrossingC2"; myCTR.Mat="outercurbT"
					local myCTL = World.GetCell(PosX-2,endY-1)
					local myRML = World.GetCell(PosX-1,endY-1)
					local myRMR = World.GetCell(PosX,endY-1)
					local myCTR = World.GetCell(PosX+1,endY-1)
					myCTL.Mat="ConcreteTiles"; myRML.Mat="RailwayCTileLeft"; myRMR.Mat="RailwayCTileRight"; myCTR.Mat="ConcreteTiles"
				elseif myCTL.Mat == "outercurbB" or myCTL.Mat == "verticalcurbwithzebraB" then
					myCTL.Mat="outercurbB"; myRML.Mat="RailwayCrossingC3"; myRMR.Mat="RailwayCrossingC4"; myCTR.Mat="outercurbB"
					local myCTL = World.GetCell(PosX-2,endY+1)
					local myRML = World.GetCell(PosX-1,endY+1)
					local myRMR = World.GetCell(PosX,endY+1)
					local myCTR = World.GetCell(PosX+1,endY+1)
					myCTL.Mat="ConcreteTiles"; myRML.Mat="RailwayCTileLeft"; myRMR.Mat="RailwayCTileRight"; myCTR.Mat="ConcreteTiles"
					endY = endY+1
					
				else
					myCTL.Mat="Gravel"; myRML.Mat="RailwayLeft"; myRMR.Mat="RailwayRight"; myCTR.Mat="Gravel"
				end
			end
		end
	else																				-- create horizontal railway
		local endX = -1
		local PosY = this.Pos.y
		local foundEndX = false
		MyChecker = Object.Spawn("RailwayTrackChecker",1,PosY)
		local CheckerPriorPosX = MyChecker.Pos.x 
		while foundEndX == false do
			endX = endX+1
			local myCell = World.GetCell(endX,PosY)
			if myCell.Mat == nil then
				foundEndX = true
				endX = endX-1
				RemoveTrees(MyChecker,4)
				MyChecker.Delete()
				if this.Or.x == 1 and this.Or.y == 0 then
					newSign = Object.Spawn("RailwayTrackSign",endX-0.5,PosY+1.5)
					newSign.Or.x = this.Or.x
					newSign.Or.y = this.Or.y
					newSign.TrackID = this.Id.u
					Set(newSign,"Tooltip","Track: "..this.Id.u)
					local myCell = World.GetCell(endX-0.5,PosY+1.5)
					-- myCell.Mat = "FX2K_Tile1"
				else
					newSign = Object.Spawn("RailwayTrackSign",1.5,PosY-1.5)
					newSign.Or.x = this.Or.x
					newSign.Or.y = this.Or.y
					newSign.TrackID = this.Id.u
					Set(newSign,"Tooltip","Track: "..this.Id.u)
					local myCell = World.GetCell(1.5,PosY-1.5)
					-- myCell.Mat = "FX2K_Tile1"
				end
			else
				MyChecker.Pos.x = endX
				if MyChecker.Pos.x >= CheckerPriorPosX+3 then
					CheckerPriorPosX = MyChecker.Pos.x
					RemoveTrees(MyChecker,4)
				end
				local myCTL = World.GetCell(endX,PosY-2)
				local myRML = World.GetCell(endX,PosY-1)
				local myRMR = World.GetCell(endX,PosY)
				local myCTR = World.GetCell(endX,PosY+1)
				
				if myCTL.Mat == "RoadMarkingsLeft" then			-- road detected
					myRML.Mat="RailwayCrossingA"; myRMR.Mat="RailwayCrossingC"
					local myCTL = World.GetCell(endX-1,PosY-2)	-- adjust the concrete tiled border to the left of the road
					local myRML = World.GetCell(endX-1,PosY-1)
					local myRMR = World.GetCell(endX-1,PosY)
					local myCTR = World.GetCell(endX-1,PosY+1)
					myCTL.Mat="ConcreteTiles"; myRML.Mat="RailwayCTileTop"; myRMR.Mat="RailwayCTileBottom"; myCTR.Mat="ConcreteTiles"
				elseif myCTL.Mat == "Road" or myCTL.Mat == "RoadMarkings" or myCTL.Mat == "dashedcenterline_vertical" or myCTL.Mat == "solidcenterline_vertical" or myCTL.Mat == "solidyellowcenterline_vertical" or myCTL.Mat == "manhole" then
					myRML.Mat="RailwayRoadTop"; myRMR.Mat="RailwayRoadBottom"
				elseif myCTL.Mat == "RoadMarkingsRight" then
					myRML.Mat="RailwayCrossingB"; myRMR.Mat="RailwayCrossingD"
					local myCTL = World.GetCell(endX+1,PosY-2)	-- adjust the concrete tiled border to the right of the road
					local myRML = World.GetCell(endX+1,PosY-1)
					local myRMR = World.GetCell(endX+1,PosY)
					local myCTR = World.GetCell(endX+1,PosY+1)
					myCTL.Mat="ConcreteTiles"; myRML.Mat="RailwayCTileTop"; myRMR.Mat="RailwayCTileBottom"; myCTR.Mat="ConcreteTiles"
					endX = endX+1
					
				elseif myCTL.Mat == "outercurbL" then			-- curb road detected
					myRML.Mat="RailwayCrossingCA"; myRMR.Mat="RailwayCrossingCC"
					local myCTL = World.GetCell(endX-1,PosY-2)	-- adjust the concrete tiled border to the left of the road
					local myRML = World.GetCell(endX-1,PosY-1)
					local myRMR = World.GetCell(endX-1,PosY)
					local myCTR = World.GetCell(endX-1,PosY+1)
					myCTL.Mat="ConcreteTiles"; myRML.Mat="RailwayCTileTop"; myRMR.Mat="RailwayCTileBottom"; myCTR.Mat="ConcreteTiles"
				elseif myCTL.Mat == "outercurbR" then
					myRML.Mat="RailwayCrossingCB"; myRMR.Mat="RailwayCrossingCD"
					local myCTL = World.GetCell(endX+1,PosY-2)	-- adjust the concrete tiled border to the right of the road
					local myRML = World.GetCell(endX+1,PosY-1)
					local myRMR = World.GetCell(endX+1,PosY)
					local myCTR = World.GetCell(endX+1,PosY+1)
					myCTL.Mat="ConcreteTiles"; myRML.Mat="RailwayCTileTop"; myRMR.Mat="RailwayCTileBottom"; myCTR.Mat="ConcreteTiles"
					endX = endX+1
					
				elseif myCTL.Mat == "RailwayLeft" then			-- vertical rails detected
					myCTL.Mat="RailwayLeft"; myRML.Mat="RailwayCrossing1"; myRMR.Mat="RailwayCrossing3"; myCTR.Mat="RailwayLeft"
					if this.Or.x == 1 and this.Or.y == 0 then
						local newSign = Object.Spawn("RailwayTrackSign",endX-0.5,PosY+1.5)
						newSign.Or.x = this.Or.x
						newSign.Or.y = this.Or.y
						newSign.TrackID = this.Id.u
						Set(newSign,"Tooltip","Track: "..this.Id.u)
						if TrackDirection(endX,PosY,"vertical") == "ToTop" then
							local newSign = Object.Spawn("RailwayTrackSign",endX+2.5,PosY+1.5)
							newSign.Or.x = 0
							newSign.Or.y = -1
							newSign.TrackID = tmpID
							Set(newSign,"Tooltip","Track: "..tmpID)
						else
							local newSign = Object.Spawn("RailwayTrackSign",endX-0.5,PosY-1.5)
							newSign.Or.x = 0
							newSign.Or.y = 1
							newSign.TrackID = tmpID
							Set(newSign,"Tooltip","Track: "..tmpID)
						end
					end
				elseif myCTL.Mat == "RailwayRight" then
					myCTL.Mat="RailwayRight"; myRML.Mat="RailwayCrossing2"; myRMR.Mat="RailwayCrossing4"; myCTR.Mat="RailwayRight"
					if this.Or.x == -1 and this.Or.y == 0 then
						local newSign = Object.Spawn("RailwayTrackSign",endX+1.5,PosY-1.5)
						newSign.Or.x = this.Or.x
						newSign.Or.y = this.Or.y
						newSign.TrackID = this.Id.u
						Set(newSign,"Tooltip","Track: "..this.Id.u)
						if TrackDirection(endX,PosY,"vertical") == "ToTop" then
							local newSign = Object.Spawn("RailwayTrackSign",endX+1.5,PosY+1.5)
							newSign.Or.x = 0
							newSign.Or.y = -1
							newSign.TrackID = tmpID
							Set(newSign,"Tooltip","Track: "..tmpID)
						else
							local newSign = Object.Spawn("RailwayTrackSign",endX-1.5,PosY-1.5)
							newSign.Or.x = 0
							newSign.Or.y = 1
							newSign.TrackID = tmpID
							Set(newSign,"Tooltip","Track: "..tmpID)
						end
					end
					
				else
					myCTL.Mat="Gravel"; myRML.Mat="RailwayTop"; myRMR.Mat="RailwayBottom"; myCTR.Mat="Gravel"
				end
			end
		end
	end
	if this.Or.x == 0 and this.Or.y == 1 then			-- train goes from top to bottom
		Direction = 1
	elseif this.Or.x == 0 and this.Or.y == -1 then		-- train goes from bottom to top
		Direction = 2
	elseif this.Or.x == 1 and this.Or.y == 0 then		-- train goes from left to right
		Direction = 3
	elseif this.Or.x == -1 and this.Or.y == 0 then		-- train goes from right to left
		Direction = 4
	end
	Interface.RemoveComponent(this,"CreateRailway")
	Interface.AddComponent(this,"toggleTrainWagons", "Button", "tooltip_button_TrainWagons",this.Wagons,"X")
	Interface.AddComponent(this,"DeleteRailway", "Button", "tooltip_button_DeleteRailway")
	for existingSpawners in next, Find(this,"TrainSpawner",10000) do
		existingSpawners.UpdateTrackSigns = true
	end
	for existingManagers in next, Find(this,"RailwayManager",10000) do
		existingManagers.UpdateTracks = true
	end
	Set(this,"Tooltip","Track: "..this.Id.u)
	Set(this,"RailwayCreated",true)
end

function TrackDirection(x,y,dir)
	local X = MyChecker.Pos.x
	local Y = MyChecker.Pos.y
	local ToDir = "None"
	if dir == "horizontal" then
		MyChecker.Pos.x = 0.5
		MyChecker.Pos.y = y
		for existingSpawners in next, Find(MyChecker,"TrainSpawner",1) do
			tmpID = existingSpawners.Id.u
			ToDir = "ToRight"
		end
		if ToDir == "None" then
			MyChecker.Pos.x = World.NumCellsX-0.5
			MyChecker.Pos.y = y
			for existingSpawners in next, Find(MyChecker,"TrainSpawner",1) do
				tmpID = existingSpawners.Id.u
				ToDir = "ToLeft"
			end
		end
	else
		MyChecker.Pos.x = x
		MyChecker.Pos.y = 0.5
		for existingSpawners in next, Find(MyChecker,"TrainSpawner",1) do
			tmpID = existingSpawners.Id.u
			ToDir = "ToBottom"
		end
		if ToDir == "None" then
			MyChecker.Pos.x = x
			MyChecker.Pos.y = World.NumCellsY-0.5
			for existingSpawners in next, Find(MyChecker,"TrainSpawner",1) do
				tmpID = existingSpawners.Id.u
				ToDir = "ToTop"
			end
		end
	end
	MyChecker.Pos.x = X
	MyChecker.Pos.y = Y
	return ToDir
end

function RemoveTrees(from,dist)
	for j = 1, #TreeTypes do
		for thatTree in next, Find(from,TreeTypes[j],dist) do
			thatTree.Delete()
		end
	end
end

function DeleteRailwayClicked()
	BuildTrackSignList()
	if #MyTrackSigns > 0 then
		for i = 1,#MyTrackSigns do
			if Exists(MyTrackSigns[i]) then
				MyTrackSigns[i].Delete()
			end
		end
	end
	
	Set(this,"RailwayCreated",nil)
	if (this.Or.x == 0 and this.Or.y == 1) or (this.Or.x == 0 and this.Or.y == -1) then	-- remove vertical railway
		local endY = -1
		local PosX = this.Pos.x
		local foundEndY = false
		MyChecker = Object.Spawn("RailwayTrackChecker",PosX,1)
		local CheckerPriorPosY = MyChecker.Pos.y
		while foundEndY == false do
			endY = endY+1
			local myCell = World.GetCell(PosX,endY)
			if myCell.Mat == nil then
				foundEndY = true
				for existingManagers in next, Find(this,"RailwayManager",10000) do
					existingManagers.UpdateTracks = true
				end
				MyChecker.Delete()
			else
				MyChecker.Pos.y = endY
				local myCTL = World.GetCell(PosX-2,endY)
				local myRML = World.GetCell(PosX-1,endY)
				local myRMR = World.GetCell(PosX,endY)
				local myCTR = World.GetCell(PosX+1,endY)
				
				if myRML.Mat == "RailwayCrossing1" then			-- horizontal rails crossing detected (top part)
					myRML.Mat="RailwayTop"; myRMR.Mat="RailwayTop"
					local myCTL = World.GetCell(PosX-2,endY-1)	-- adjust the gravel tiled border to the top of the horizontal rails
					local myRML = World.GetCell(PosX-1,endY-1)
					local myRMR = World.GetCell(PosX,endY-1)
					local myCTR = World.GetCell(PosX+1,endY-1)
					myCTL.Mat="Gravel"; myRML.Mat="Gravel"; myRMR.Mat="Gravel"; myCTR.Mat="Gravel"
					if this.Or.x == 0 and this.Or.y == 1 then
						if TrackDirection(PosX,endY,"horizontal") == "ToLeft" then
							RemoveRailwayTrackSign(PosX+1.5,endY-0.5,tmpID)
						else
							RemoveRailwayTrackSign(PosX-1.5,endY+2.5,tmpID)
						end
					end
				elseif myRML.Mat == "RailwayCrossing3" then			-- horizontal rails crossing detected (bottom part)
					myRML.Mat="RailwayBottom"; myRMR.Mat="RailwayBottom"
					if this.Or.x == 0 and this.Or.y == -1 then
						if TrackDirection(PosX,endY,"horizontal") == "ToLeft" then
							RemoveRailwayTrackSign(PosX+1.5,endY-1.5,tmpID)
						else
							RemoveRailwayTrackSign(PosX-1.5,endY+1.5,tmpID)
						end
					end
					local myCTL = World.GetCell(PosX-2,endY+1)	-- adjust the gravel tiled border at the bottom of the horizontal rails
					local myRML = World.GetCell(PosX-1,endY+1)
					local myRMR = World.GetCell(PosX,endY+1)
					local myCTR = World.GetCell(PosX+1,endY+1)
					myCTL.Mat="Gravel"; myRML.Mat="Gravel"; myRMR.Mat="Gravel"; myCTR.Mat="Gravel"
					endY = endY+1					
					
				elseif myCTL.Mat == "outerlineT" or myCTL.Mat == "verticallinewithzebraT" then		-- horizontal road detected which was created with "outerlineT"
					local myCTL = World.GetCell(PosX-2,endY-1)
					local myRML = World.GetCell(PosX-1,endY-1)
					local myRMR = World.GetCell(PosX,endY-1)
					local myCTR = World.GetCell(PosX+1,endY-1)
					myCTL.Mat="ConcreteTiles"; myRML.Mat="ConcreteTiles"; myRMR.Mat="ConcreteTiles"; myCTR.Mat="ConcreteTiles"
					local myCTL = World.GetCell(PosX-2,endY)
					local myRML = World.GetCell(PosX-1,endY)
					local myRMR = World.GetCell(PosX,endY)
					local myCTR = World.GetCell(PosX+1,endY)
					myCTL.Mat="outerlineT"; myRML.Mat="outerlineT"; myRMR.Mat="outerlineT"; myCTR.Mat="outerlineT"
					
				elseif myCTL.Mat == "Road" or myCTL.Mat == "verticalzebra" or myCTL.Mat == "dashedcenterline_horizontal" or myCTL.Mat == "solidcenterline_horizontal" or myCTL.Mat == "solidyellowcenterline_horizontal" or myCTL.Mat == "manhole" then
					myCTL.Mat="Road"; myRML.Mat="Road"; myRMR.Mat="Road"; myCTR.Mat="Road"
					
				elseif myCTL.Mat == "outerlineB" or myCTL.Mat == "verticallinewithzebraB" then
					myCTL.Mat="outerlineB"; myRML.Mat="outerlineB"; myRMR.Mat="outerlineB"; myCTR.Mat="outerlineB"
					local myCTL = World.GetCell(PosX-2,endY+1)
					local myRML = World.GetCell(PosX-1,endY+1)
					local myRMR = World.GetCell(PosX,endY+1)
					local myCTR = World.GetCell(PosX+1,endY+1)
					myCTL.Mat="ConcreteTiles"; myRML.Mat="ConcreteTiles"; myRMR.Mat="ConcreteTiles"; myCTR.Mat="ConcreteTiles"
					endY = endY+1					
					
				elseif myCTL.Mat == "outercurbT" or myCTL.Mat == "verticalcurbwithzebraT" or myCTL.Mat == "curbgutter" then		-- horizontal road detected which was created with "outercurbT"
					local myCTL = World.GetCell(PosX-2,endY-1)
					local myRML = World.GetCell(PosX-1,endY-1)
					local myRMR = World.GetCell(PosX,endY-1)
					local myCTR = World.GetCell(PosX+1,endY-1)
					myCTL.Mat="ConcreteTiles"; myRML.Mat="ConcreteTiles"; myRMR.Mat="ConcreteTiles"; myCTR.Mat="ConcreteTiles"
					local myCTL = World.GetCell(PosX-2,endY)
					local myRML = World.GetCell(PosX-1,endY)
					local myRMR = World.GetCell(PosX,endY)
					local myCTR = World.GetCell(PosX+1,endY)
					myCTL.Mat="outercurbT"; myRML.Mat="outercurbT"; myRMR.Mat="outercurbT"; myCTR.Mat="outercurbT"
				elseif myCTL.Mat == "outercurbB" or myCTL.Mat == "verticalcurbwithzebraB" then
					myCTL.Mat="outercurbB"; myRML.Mat="outercurbB"; myRMR.Mat="outercurbB"; myCTR.Mat="outercurbB"
					local myCTL = World.GetCell(PosX-2,endY+1)
					local myRML = World.GetCell(PosX-1,endY+1)
					local myRMR = World.GetCell(PosX,endY+1)
					local myCTR = World.GetCell(PosX+1,endY+1)
					myCTL.Mat="ConcreteTiles"; myRML.Mat="ConcreteTiles"; myRMR.Mat="ConcreteTiles"; myCTR.Mat="ConcreteTiles"
					endY = endY+1
					
				else
					myCTL.Mat="Dirt"; myRML.Mat="Dirt"; myRMR.Mat="Dirt"; myCTR.Mat="Dirt"
				end
			end
		end
	else																				-- delete horizontal railway
		local endX = -1
		local PosY = this.Pos.y
		local foundEndX = false
		MyChecker = Object.Spawn("RailwayTrackChecker",1,PosY)
		local CheckerPriorPosX = MyChecker.Pos.x 
		while foundEndX == false do
			endX = endX+1
			local myCell = World.GetCell(endX,PosY)
			if myCell.Mat == nil then
				foundEndX = true
				for existingManagers in next, Find(this,"RailwayManager",10000) do
					existingManagers.UpdateTracks = true
				end
				MyChecker.Delete()
			else
				MyChecker.Pos.x = endX
				local myCTL = World.GetCell(endX,PosY-2)
				local myRML = World.GetCell(endX,PosY-1)
				local myRMR = World.GetCell(endX,PosY)
				local myCTR = World.GetCell(endX,PosY+1)
				
				if myCTL.Mat == "RoadMarkingsLeft" then			-- road detected
					myRML.Mat="RoadMarkingsLeft"; myRMR.Mat="RoadMarkingsLeft"
					local myCTL = World.GetCell(endX-1,PosY-2)	-- adjust the concrete tiled border to the left of the road
					local myRML = World.GetCell(endX-1,PosY-1)
					local myRMR = World.GetCell(endX-1,PosY)
					local myCTR = World.GetCell(endX-1,PosY+1)
					myCTL.Mat="ConcreteTiles"; myRML.Mat="ConcreteTiles"; myRMR.Mat="ConcreteTiles"; myCTR.Mat="ConcreteTiles"
					
				elseif myCTL.Mat == "Road" or myCTL.Mat == "RoadMarkings" or myCTL.Mat == "dashedcenterline_vertical" or myCTL.Mat == "solidcenterline_vertical" or myCTL.Mat == "solidyellowcenterline_vertical" or myCTL.Mat == "manhole" then
					myRML.Mat="Road"; myRMR.Mat="Road"
				elseif myCTL.Mat == "RoadMarkingsRight" then
					myRML.Mat="RoadMarkingsRight"; myRMR.Mat="RoadMarkingsRight"
					local myCTL = World.GetCell(endX+1,PosY-2)	-- adjust the concrete tiled border to the right of the road
					local myRML = World.GetCell(endX+1,PosY-1)
					local myRMR = World.GetCell(endX+1,PosY)
					local myCTR = World.GetCell(endX+1,PosY+1)
					myCTL.Mat="ConcreteTiles"; myRML.Mat="ConcreteTiles"; myRMR.Mat="ConcreteTiles"; myCTR.Mat="ConcreteTiles"
					endX = endX+1
					
				elseif myCTL.Mat == "outercurbL" then			-- curb road detected
					myRML.Mat="outercurbL"; myRMR.Mat="outercurbL"
					local myCTL = World.GetCell(endX-1,PosY-2)	-- adjust the concrete tiled border to the left of the road
					local myRML = World.GetCell(endX-1,PosY-1)
					local myRMR = World.GetCell(endX-1,PosY)
					local myCTR = World.GetCell(endX-1,PosY+1)
					myCTL.Mat="ConcreteTiles"; myRML.Mat="ConcreteTiles"; myRMR.Mat="ConcreteTiles"; myCTR.Mat="ConcreteTiles"
				elseif myCTL.Mat == "outercurbR" then
					myRML.Mat="outercurbR"; myRMR.Mat="outercurbR"
					local myCTL = World.GetCell(endX+1,PosY-2)	-- adjust the concrete tiled border to the right of the road
					local myRML = World.GetCell(endX+1,PosY-1)
					local myRMR = World.GetCell(endX+1,PosY)
					local myCTR = World.GetCell(endX+1,PosY+1)
					myCTL.Mat="ConcreteTiles"; myRML.Mat="ConcreteTiles"; myRMR.Mat="ConcreteTiles"; myCTR.Mat="ConcreteTiles"
					endX = endX+1
					
				elseif myCTL.Mat == "RailwayLeft" then			-- vertical rails detected
					myCTL.Mat="RailwayLeft"; myRML.Mat="RailwayLeft"; myRMR.Mat="RailwayLeft"; myCTR.Mat="RailwayLeft"
					local myCTL = World.GetCell(endX-1,PosY-2)	-- adjust the gravel tiled border to the left of the rails
					local myRML = World.GetCell(endX-1,PosY-1)
					local myRMR = World.GetCell(endX-1,PosY)
					local myCTR = World.GetCell(endX-1,PosY+1)
					myCTL.Mat="Gravel"; myRML.Mat="Gravel"; myRMR.Mat="Gravel"; myCTR.Mat="Gravel"
					if this.Or.x == 1 and this.Or.y == 0 then
						if TrackDirection(endX,PosY,"vertical") == "ToTop" then
							RemoveRailwayTrackSign(endX+2.5,PosY+1.5,tmpID)
						else
							RemoveRailwayTrackSign(endX-0.5,PosY-1.5,tmpID)
						end
					end
					
				elseif myCTL.Mat == "RailwayRight" then
					myCTL.Mat="RailwayRight"; myRML.Mat="RailwayRight"; myRMR.Mat="RailwayRight"; myCTR.Mat="RailwayRight"
					if this.Or.x == -1 and this.Or.y == 0 then
						if TrackDirection(endX,PosY,"vertical") == "ToTop" then
							RemoveRailwayTrackSign(endX+1.5,PosY+1.5,tmpID)
						else
							RemoveRailwayTrackSign(endX-1.5,PosY-1.5,tmpID)
						end
					end
					local myCTL = World.GetCell(endX+1,PosY-2)	-- adjust the gravel tiled border to the right of the rails
					local myRML = World.GetCell(endX+1,PosY-1)
					local myRMR = World.GetCell(endX+1,PosY)
					local myCTR = World.GetCell(endX+1,PosY+1)
					myCTL.Mat="Gravel"; myRML.Mat="Gravel"; myRMR.Mat="Gravel"; myCTR.Mat="Gravel"
					endX = endX+1
					
				else
					myCTL.Mat="Dirt"; myRML.Mat="Dirt"; myRMR.Mat="Dirt"; myCTR.Mat="Dirt"
				end
			end
		end
	end
	this.Delete()
end

function RemoveRailwayTrackSign(x,y,ID)
	local X = MyChecker.Pos.x
	local Y = MyChecker.Pos.y
	MyChecker.Pos.x = x
	MyChecker.Pos.y = y
	for existingTrackSigns in next, Find(MyChecker,"RailwayTrackSign",1) do
		if existingTrackSigns.TrackID == ID then existingTrackSigns.Delete() end
	end
	MyChecker.Pos.x = X
	MyChecker.Pos.y = Y
end
	
function Exists(theObject)
	if theObject ~= nil and theObject.SubType ~= nil then
		return true
	else
		return false
	end
end
